# Generated by Django 2.1.4 on 2019-01-13 03:07

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="User",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("password", models.CharField(max_length=128, verbose_name="password")),
                (
                    "last_login",
                    models.DateTimeField(blank=True, null=True, verbose_name="last login"),
                ),
                ("email", models.EmailField(max_length=254, unique=True)),
                ("display_name", models.CharField(max_length=50)),
            ],
            options={
                "abstract": False,
            },
        ),
    ]
