<dl>
    <dt>Path:</dt>
    <dd>{{ path }}</dd>
    <dt>User:</dt>
    <dd>{{ user }}</dd>
    <dt>Schema:</dt>
    <dd>{{ schema }}</dd>
    <dt>Routing:</dt>
    <dd>{{ routing }}</dd>
    <dt>Admin URL:</dt>
    <dd>{{ admin_url }}</dd>
</dl>
